# -*- py_file.py: python ; coding: utf-8 -*-
P_3_ = ":/pic/3_.png"
P_3DOTS_H = ":/pic/3dots_h.png"
P_3DOTS_V = ":/pic/3dots_v.png"
P_3I = ":/pic/3i.png"
P_4_DOTS = ":/pic/4_dots.png"
P_4_SQUARE = ":/pic/4_square.png"
P_ADD_SUBS = ":/pic/add_subs.png"
P_ADDGREEN = ":/pic/addgreen.png"
P_APPICON = ":/pic/appicon.png"
P_AR_ARROW = ":/pic/ar_arrow.png"
P_AR_CHECK = ":/pic/ar_check.png"
P_ARROW_CURSOR = ":/pic/arrow_cursor.png"
P_ATTACH = ":/pic/attach.png"
P_BACK = ":/pic/back.png"
P_BALL = ":/pic/ball.png"
P_BALL2 = ":/pic/ball2.png"
P_BALL_BLUE = ":/pic/ball_blue.png"
P_BALL_BULLET = ":/pic/ball_bullet.png"
P_BALL_DARKRED = ":/pic/ball_darkred.png"
P_BALL_GOLDEN = ":/pic/ball_golden.png"
P_BALL_GREEN = ":/pic/ball_green.png"
P_BALL_LIGHT = ":/pic/ball_light.png"
P_BALL_RED = ":/pic/ball_red.png"
P_BIZ_1 = ":/pic/biz_1.png"
P_BIZ_2 = ":/pic/biz_2.png"
P_BLUE_STAR = ":/pic/blue_star.png"
P_BULB = ":/pic/bulb.png"
P_BULLET1 = ":/pic/bullet1.png"
P_BULLET1_GOLDEN = ":/pic/bullet1_golden.png"
P_BULLET2 = ":/pic/bullet2.png"
P_BULLET2_GOLDEN = ":/pic/bullet2_golden.png"
P_BULLET3 = ":/pic/bullet3.png"
P_BULLET3_GOLDEN = ":/pic/bullet3_golden.png"
P_BULLET4 = ":/pic/bullet4.png"
P_CALC = ":/pic/calc.png"
P_CALENDAR = ":/pic/calendar.png"
P_CANCEL = ":/pic/cancel.png"
P_CHART = ":/pic/chart.png"
P_CHART_BAR = ":/pic/chart_bar.png"
P_CHART_LINE = ":/pic/chart_line.png"
P_CHART_LINEBAR = ":/pic/chart_linebar.png"
P_CHART_PIE = ":/pic/chart_pie.png"
P_CHAT = ":/pic/chat.png"
P_CHECK = ":/pic/check.png"
P_CHECK_SHEET = ":/pic/check_sheet.png"
P_CIRCLE_CUBE = ":/pic/circle_cube.png"
P_CIRCLE_CUBE2 = ":/pic/circle_cube2.png"
P_CLIP_BOARD = ":/pic/clip_board.png"
P_CLOSE = ":/pic/close.png"
P_CLOSE_X = ":/pic/close_x.png"
P_CONTINUE = ":/pic/continue.png"
P_CONTRACT = ":/pic/contract.png"
P_CONTRACT_2 = ":/pic/contract_2.png"
P_CUBE4 = ":/pic/cube4.png"
P_CUSTOM = ":/pic/custom.png"
P_CUSTOM_CARE = ":/pic/custom_care.png"
P_CUSTOM_MONEY = ":/pic/custom_money.png"
P_DB = ":/pic/db.png"
P_DCUBE = ":/pic/dcube.png"
P_DIAMOND = ":/pic/diamond.png"
P_DOC = ":/pic/doc.png"
P_DOC2 = ":/pic/doc2.png"
P_DOC_PERCENT = ":/pic/doc_percent.png"
P_DOC_RPT = ":/pic/doc_rpt.png"
P_DOWNLOAD = ":/pic/download.png"
P_DOWNLOAD_2 = ":/pic/download_2.png"
P_DROP = ":/pic/drop.png"
P_EARTH = ":/pic/earth.png"
P_EXEC_ALL = ":/pic/exec_all.png"
P_EXECUTE = ":/pic/execute.png"
P_EXPORT = ":/pic/export.png"
P_EYE = ":/pic/eye.png"
P_FANGSHENG = ":/pic/fangsheng.png"
P_FAPICON = ":/pic/fapicon.png"
P_FILTER = ":/pic/filter.png"
P_FIT_WIDTH = ":/pic/fit_width.png"
P_FLY_UP = ":/pic/fly_up.png"
P_FORWARD = ":/pic/forward.png"
P_FUNC = ":/pic/func.png"
P_FUNC_GROUP = ":/pic/func_group.png"
P_FUNC_ITEM = ":/pic/func_item.png"
P_FUNCTION = ":/pic/function.png"
P_GAUGE = ":/pic/gauge.png"
P_GOAL = ":/pic/goal.png"
P_GOLDEN_STAR = ":/pic/golden_star.png"
P_GREEN_STAR = ":/pic/green_star.png"
P_GRID = ":/pic/grid.png"
P_GRID_1 = ":/pic/grid_1.png"
P_GRID_CUBE = ":/pic/grid_cube.png"
P_GRIDVIEW = ":/pic/gridview.png"
P_HAND = ":/pic/hand.png"
P_HEART = ":/pic/heart.png"
P_HEXSTAR = ":/pic/hexstar.png"
P_HOME = ":/pic/home.png"
P_HTML = ":/pic/html.png"
P_INDEX = ":/pic/index.png"
P_INOUT = ":/pic/inout.png"
P_JOIN = ":/pic/join.png"
P_KEY = ":/pic/key.png"
P_LABEL = ":/pic/label.png"
P_LABEL_2 = ":/pic/label_2.png"
P_LAST = ":/pic/last.png"
P_LEFT = ":/pic/left.png"
P_LEFT2 = ":/pic/left2.png"
P_LEVELS = ":/pic/levels.png"
P_LIGHTNING = ":/pic/lightning.png"
P_LINK = ":/pic/link.png"
P_LOCK = ":/pic/lock.png"
P_LOGO = ":/pic/logo.png"
P_LOOKUP = ":/pic/lookup.png"
P_LOOP = ":/pic/loop.png"
P_MAX = ":/pic/max.png"
P_MBLOCKS = ":/pic/mblocks.png"
P_MILESTONE = ":/pic/milestone.png"
P_MIN = ":/pic/min.png"
P_MINUS = ":/pic/minus.png"
P_MONEY_FORWARD = ":/pic/money_forward.png"
P_MORE = ":/pic/more.png"
P_MULTI_BLOCK = ":/pic/multi_block.png"
P_NEXT = ":/pic/next.png"
P_NOTEBOOK = ":/pic/notebook.png"
P_NSTC = ":/pic/nstc.png"
P_OFF = ":/pic/off.png"
P_OKR = ":/pic/okr.png"
P_OPEN = ":/pic/open.png"
P_ORG = ":/pic/org.png"
P_ORG1 = ":/pic/org1.png"
P_ORG2 = ":/pic/org2.png"
P_OUTCOME = ":/pic/outcome.png"
P_PAPER_PLANE = ":/pic/paper_plane.png"
P_PARTS = ":/pic/parts.png"
P_PAUSE = ":/pic/pause.png"
P_PC = ":/pic/pc.png"
P_PC_BAR = ":/pic/pc_bar.png"
P_PERSON_CARD = ":/pic/person_card.png"
P_PERSON_CHART = ":/pic/person_chart.png"
P_PERSON_DOC = ":/pic/person_doc.png"
P_PERSON_FIRE = ":/pic/person_fire.png"
P_PERSON_HIRE = ":/pic/person_hire.png"
P_PIE = ":/pic/pie.png"
P_PIE_2 = ":/pic/pie_2.png"
P_PLAY = ":/pic/play.png"
P_PLUS = ":/pic/plus.png"
P_RED_STAR = ":/pic/red_star.png"
P_REFRESH = ":/pic/refresh.png"
P_REPORT = ":/pic/report.png"
P_REVERT = ":/pic/revert.png"
P_RIGHT = ":/pic/right.png"
P_RIGHT2 = ":/pic/right2.png"
P_RIGHT_3 = ":/pic/right_3.png"
P_SAVE = ":/pic/save.png"
P_SAVE_DATA = ":/pic/save_data.png"
P_SETTING = ":/pic/setting.png"
P_SHOW_NORMAL = ":/pic/show_normal.png"
P_SITE = ":/pic/site.png"
P_SITE_BLUE = ":/pic/site_blue.png"
P_SORT = ":/pic/sort.png"
P_STAR = ":/pic/star.png"
P_STOP = ":/pic/stop.png"
P_SUM = ":/pic/sum.png"
P_SUM_1 = ":/pic/sum_1.png"
P_TARGET = ":/pic/target.png"
P_TIME = ":/pic/time.png"
P_TO_CLOUD = ":/pic/to_cloud.png"
P_TODO = ":/pic/todo.png"
P_TOOLS = ":/pic/tools.png"
P_TREE = ":/pic/tree.png"
P_TRUSH = ":/pic/trush.png"
P_TUMBER = ":/pic/tumber.png"
P_TURN_UP = ":/pic/turn_up.png"
P_TWO_PERSONS = ":/pic/two_persons.png"
P_USER = ":/pic/user.png"
P_VAR = ":/pic/var.png"
P_VIEW = ":/pic/view.png"
P_VV_DOWN = ":/pic/vv_down.png"
P_VVV_DOWN = ":/pic/vvv_down.png"
I_AIRPLANE = ":/icon/airplane.ico"
I_CALC = ":/icon/calc.ico"
I_CIRCLE_CUBE2 = ":/icon/circle_cube2.ico"
I_DCUBE = ":/icon/dcube.ico"
I_FAP = ":/icon/fap.ico"
I_NDAS = ":/icon/ndas.ico"
I_NDW = ":/icon/ndw.ico"

